from typing import Optional, Tuple, Any, Dict, Union
from logging import getLogger
from requests import post, RequestException

from watchmen_cli.common.constants import login_url, access_token
from watchmen_cli.common.exception import AuthenticationException


logger = getLogger(__name__)


class Client:

    def __init__(self, host: str, pat: Optional[str], username: Optional[str], password: Optional[str]):
        self.host = host
        self.pat = pat
        self.username = username
        self.password = password

    def login(self) -> str:
        token = None
        if self.pat:
            token = f"pat {self.pat}"
            return token
        else:
            if self.username and self.password:
                login_data = {'username': self.username, 'password': self.password}
                headers = {'Content-Type': 'application/x-www-form-urlencoded'}
                status, result = self.post(login_url, login_data, headers)
                token = result.get(access_token, None)
        if token:
            return f"Bearer {token}"
        else:
            raise AuthenticationException("Need Pat or UserName/Password")

    def post(self, url: str, data: Optional[Union[Dict, str]], headers: Optional[Dict],
             params: Optional[Dict] = None) -> Tuple[int, Any]:
        try:
            response = post(self.host_url(url), data=data, params=params, headers=headers)
            return response.status_code, response.json()
        except RequestException as e:
            logger.error(e, exc_info=True)
            return 0, None

    def host_url(self, url) -> str:
        return f"{self.host}{url}"



from enum import Enum


class MixedImportType(str, Enum):
    NON_REDUNDANT = 'non-redundant'
    REPLACE = 'replace'
    FORCE_NEW = 'force-new'


NON_REDUNDANT = 'non-redundant'
REPLACE = 'replace'
FORCE_NEW = 'force-new'

login_url = '/login'
access_token = 'accessToken'

pipeline_id = 'pipelineId'
topic_id = 'topicId'
space_id = 'spaceId'
connect_id = 'connectId'

topics = 'topics'
pipelines = 'pipelines'
spaces = 'spaces'
connected_spaces = 'connectedSpaces'
import_type = 'importType'

prefix_encoding = '<a href="data:application/json;base64,'





from typing import Optional

from watchmen_cli.common.constants import MixedImportType
from watchmen_cli.common.exception import ConfigException
from watchmen_cli.service import Deployment
from watchmen_cli.settings import settings


class CommandLineInterfaces:

    @staticmethod
    def deploy_asset():
        if settings.META_CLI_PAT is None and (settings.META_CLI_USERNAME is None or settings.META_CLI_PASSWORD is None):
            raise ConfigException("must have authentication configuration")
        if settings.META_CLI_DEPLOY_FOLDER is None:
            raise ConfigException("must have data path configuration")
        dm = Deployment(settings.META_CLI_HOST, settings.META_CLI_PAT,
                        settings.META_CLI_USERNAME, settings.META_CLI_PASSWORD,
                        settings.META_CLI_DEPLOY_FOLDER, settings.META_CLI_DEPLOY_PATTERN)
        dm.deploy()

    @staticmethod
    def deploy(host: Optional[str],
               path: Optional[str],
               pattern: Optional[MixedImportType] = 'replace',
               username: Optional[str] = None,
               password: Optional[str] = None,
               pat: Optional[str] = None,
               ):
        if pat is None and (username is None or password is None):
            raise ConfigException("must have authentication configuration")
        if path is None:
            raise ConfigException("must have data path configuration")
        dm = Deployment(host, pat, username, password, path, pattern)
        dm.deploy()



from logging import getLogger
from pydantic import BaseSettings

from watchmen_cli.common.constants import REPLACE

logger = getLogger(__name__)


class CliSettings(BaseSettings):

	META_CLI_HOST: str = 'http://localhost'
	META_CLI_USERNAME: str = None
	META_CLI_PASSWORD: str = None
	META_CLI_PAT: str = None
	META_CLI_DEPLOY_FOLDER:  str = None
	META_CLI_DEPLOY_PATTERN: str = REPLACE

	class Config:
		# secrets_dir = '/var/run'
		env_file = '.env'
		env_file_encoding = 'utf-8'
		case_sensitive = True


settings = CliSettings()
logger.info(f'Cli settings[{settings.dict()}].')



import os
from typing import Tuple, List, Optional

from watchmen_cli.common.client import Client
from watchmen_cli.common.constants import topics, pipelines, spaces, connected_spaces, import_type, pipeline_id, \
    connect_id, space_id, topic_id, prefix_encoding, MixedImportType

from watchmen_cli.common.exception import DeployException
from watchmen_utilities import ArrayHelper
from base64 import b64decode
from json import loads, dumps
from logging import getLogger

logger = getLogger(__name__)


class Deployment:

    def __init__(self, host: str, pat: Optional[str],
                 username: Optional[str], password: Optional[str],
                 path: Optional[str], pattern: Optional[MixedImportType]):
        self.host = host
        self.pat = pat
        self.username = username
        self.password = password
        self.deploy_folder = path
        self.pattern = pattern

    def get_deploy_file(self) -> str:
        for root, ds, fns in os.walk(self.deploy_folder):
            for fn in fns:
                if fn.endswith('.md'):
                    fullname = os.path.join(root, fn)
                    yield fullname

    def deploy(self):
        for file_name in self.get_deploy_file():
            self.data_parse(file_name)

    @staticmethod
    def read_data_from_markdown(file_content: str) -> Tuple[List, List, List, List]:
        topic_list = []
        pipeline_list = []
        space_list = []
        connect_space_list = []

        json_list = ArrayHelper(file_content.split("\n")) \
            .map(lambda x: x.strip()) \
            .filter(lambda x: x.startswith(prefix_encoding)) \
            .map(lambda x: x.replace(prefix_encoding, '')) \
            .map(lambda x: b64decode(x[0:x.find('"')])) \
            .map(lambda x: loads(x)) \
            .to_list()

        for json_data in json_list:
            if pipeline_id in json_data:
                pipeline_list.append(json_data)
            elif topic_id in json_data:
                topic_list.append(json_data)
            elif space_id in json_data and connect_id not in json_data:
                space_list.append(json_data)
            elif connect_id in json_data:
                connect_space_list.append(json_data)
        return topic_list, pipeline_list, space_list, connect_space_list

    def data_parse(self, fullname: str):
        with open(fullname, encoding='utf-8') as file:
            logger.info(f'Deploy asset fullname: {fullname}')
            data = file.read()
            topic_list, pipeline_list, space_list, connect_space_list = self.read_data_from_markdown(data)
            data_ = {
                topics: topic_list,
                pipelines: pipeline_list,
                spaces: space_list,
                connected_spaces: connect_space_list,
                import_type: self.pattern
            }
            self.import_md_asset(data_)

    def import_md_asset(self, data) -> int:
        client = Client(self.host, self.pat, self.username, self.password)
        token = client.login()
        headers = {'Content-Type': 'application/json', 'Authorization': token}
        status, result = client.post('/import', data=dumps(data), headers=headers)
        if status == 200:
            logger.info('Deploy asset successful.')
        else:
            raise DeployException(f"Import md asset failed. Error: {result}")
